@@ -16,10 +16,11 @@ use DBI;
 use File::Spec;
 use Data::Identifier;
 use Data::Identifier::Generate;
+use Scalar::Util qw(weaken);
 
 use File::FStore::File;
 
-our $VERSION = v0.03;
+our $VERSION = v0.04;
 
 my %_types = (
     db          => 'Data::TagDB',
@@ -40,7 +41,7 @@ sub create {
     my %extra;
     my $dbh;
 
-    foreach my $key (qw(db extractor)) {
+    foreach my $key (keys(%_types), qw(weak)) {
         my $v = delete $opts{$key};
         $extra{$key} = $v if defined $v;
     }
@@ -92,6 +93,7 @@ SQL
 sub new {
     my ($pkg, %opts) = @_;
     my $path = delete $opts{path} // croak 'No path given';
+    my $weak = delete $opts{weak};
     my @used_digests;
     my $self = bless {
         path => $path,
@@ -104,6 +106,7 @@ sub new {
         next unless defined $v;
         croak 'Invalid type for key: '.$key unless eval {$v->isa($_types{$key})};
         $self->{$key} = $v;
+        weaken($self->{$key}) if $weak;
     }
 
     croak 'Stray options passed' if scalar keys %opts;
@@ -361,7 +364,7 @@ sub scan {
     my ($self, %opts) = @_;
     my %extra;
 
-    foreach my $key (qw(update no_digests)) {
+    foreach my $key (qw(update no_digests on_pre_set on_post_set)) {
         $extra{$key} = delete $opts{$key};
     }
 
@@ -393,13 +396,15 @@ sub fix {
     my ($self, @fixes) = @_;
     my %fixes = map {$_ => 1} @fixes;
 
-    foreach my $fix (qw(scrub remove-inode remove-mediasubtype remove-inodeise scan)) {
+    foreach my $fix (qw(upgrade scrub remove-inode remove-mediasubtype remove-inodeise scan)) {
         next unless delete $fixes{$fix};
 
         if ($fix eq 'scrub') {
             $self->scrub;
         } elsif ($fix eq 'scan') {
             $self->scan;
+        } elsif ($fix eq 'upgrade') {
+            $self->migration->upgrade;
         } elsif ($fix =~ /^remove-(inode|mediasubtype|inodeise)$/) {
             my $what = $1;
             my $sth = $self->{dbh}->prepare('DELETE FROM file_properties WHERE key = ?');
@@ -415,6 +420,7 @@ sub fix {
 }
 
 
+#@returns File::FStore::Adder
 sub new_adder {
     my ($self) = @_;
     require File::FStore::Adder;
@@ -422,6 +428,14 @@ sub new_adder {
 }
 
 
+#@returns File::FStore::Migration
+sub migration {
+    my ($self) = @_;
+    require File::FStore::Migration;
+    return File::FStore::Migration->_new(store => $self);
+}
+
+
 sub export {
     my ($self, $handle, %opts) = @_;
     my $format = delete($opts{format}) // 'json';
@@ -511,6 +525,23 @@ sub import_data {
 }
 
 
+sub attach {
+    my ($self, %opts) = @_;
+    my $weak = delete $opts{weak};
+
+    foreach my $key (keys %_types) {
+        my $v = delete $opts{$key};
+        next unless defined $v;
+        croak 'Invalid type for key: '.$key unless eval {$v->isa($_types{$key})};
+        $self->{$key} //= $v;
+        croak 'Missmatch for key: '.$key unless $self->{$key} == $v;
+        weaken($self->{$key}) if $weak;
+    }
+
+    croak 'Stray options passed' if scalar keys %opts;
+}
+
+
 #@returns Data::TagDB
 sub db {
     my ($self, %opts) = @_;
@@ -611,17 +642,19 @@ sub _scan_file {
     $file->_open(fh => $fh);
 
     if ($update eq 'all' || ($new && $update eq 'new')) {
-        $file->update(%opts{qw(no_digests)});
+        $file->update(%opts{qw(no_digests on_pre_set on_post_set)});
     } else {
         my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
             $atime,$mtime,$ctime,$blksize,$blocks) = $file->stat;
 
+        $opts{on_pre_set}->($file) if defined $opts{on_pre_set};
         $file->set({
                 properties => {
                     size => $size,
                     inode => $ino,
                 },
             });
+        $opts{on_post_set}->($file) if defined $opts{on_post_set};
     }
 }
 
@@ -639,7 +672,7 @@ File::FStore - Module for interacting with file stores
 
 =head1 VERSION
 
-version v0.03
+version v0.04
 
 =head1 SYNOPSIS
 
@@ -711,6 +744,11 @@ A L<File::Information> object. See L</fii>.
 
 The path to the store.
 
+=item C<weak>
+
+Marks the value for C<db>, C<extractor>, C<fii> as weak.
+If only a specific one needs needs to be weaken use L</attach>.
+
 =back
 
 =head2 close
@@ -873,6 +911,14 @@ This determines whether L<File::FStore::File/update> is called on all files, onl
 
 Passed to L<File::FStore::File/update>.
 
+=item C<on_pre_set>
+
+See L<File::FStore::File/update>. Also called on each file if no update is performed.
+
+=item C<on_post_set>
+
+See L<File::FStore::File/update>. Also called on each file if no update is performed.
+
 =back
 
 =head2 fix
@@ -918,6 +964,10 @@ Runs L</scan> with default options.
 
 Runs L</scrub>.
 
+=item C<upgrade>
+
+Runs L<File::FStore::Migration/upgrade>.
+
 =back
 
 =head2 new_adder
@@ -926,6 +976,12 @@ Runs L</scrub>.
 
 Create a new L<File::FStore::Adder>.
 
+=head2 migration
+
+    my File::FStore::Migration $migration = $self->migration;
+
+Returns a new L<File::FStore::Migration> object for this store.
+
 =head2 export
 
     $store->export($handle, %opts);
@@ -976,6 +1032,19 @@ The format to use. Currently supported is C<json> for the classic JSON format.
 
 =back
 
+=head2 attach
+
+    $self->attach(key => $obj, ...);
+    # or:
+    $self->attach(key => $obj, ..., weak => 1);
+
+Attaches objects of the given type.
+Takes the same list of objects as L</new>.
+
+If an object is allready attached for the given key this method C<die>s unless the object is actually the same.
+
+If C<weak> is set to a true value the object reference becomes weak.
+
 =head2 db
 
     my Data::TagDB $db = $store->db;
